@@ -7,7 +7,7 @@ use diagnostics;
 use mro 'c3';
 use English qw(-no_match_vars);
 use Carp;
-our $VERSION = 6.1;
+our $VERSION = 6.2;
 use Fatal qw( close );
 use Array::Contains;
 #---AUTOPRAGMAEND---
@@ -111,6 +111,10 @@ sub init {
         $self->{persistance} = 0;
     }
 
+    if(!defined($self->{config}->{interclacksreconnecttimeout})) {
+        $self->{config}->{interclacksreconnecttimeout} = 30;
+    }
+
     my @tcpsockets;
 
     foreach my $ip (@{$config->{ip}}) {
@@ -157,6 +161,7 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
     my $shutdowntime;
     my $selector = IO::Select->new();
     my $interclackslock = 0;
+    my $nextinterclackscheck = 0;
 
     my $keepRunning = 1;
     $SIG{INT} = sub { $keepRunning = 0; };
@@ -209,7 +214,9 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
         if(defined($self->{config}->{master})) {
             # We are in client mode. We need to add an interclacks link
             my $mcid = $self->{config}->{master}->{ip}->[0] . ':' . $self->{config}->{master}->{port};
-            if(!defined($clients{$mcid})) {
+            if(!defined($clients{$mcid}) && $nextinterclackscheck < time) {
+                $nextinterclackscheck = time + $self->{config}->{interclacksreconnecttimeout} + int(rand(10));
+
                 print "Connect to master\n";
                 my $msocket = IO::Socket::IP->new(
                     PeerHost => $self->{config}->{master}->{ip}->[0],
@@ -479,8 +486,11 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
                             if($value) {
                                 print "Interclacks sync lock ON.\n";
                                 $interclackslock = 1;
-                                
-                                # Send server our keys to the server
+                            } else {
+                                print "Interclacks sync lock OFF.\n";
+                                $interclackslock = 0;
+
+                                # Send server our keys AFTER we got everything FROM the server (e.g. after unlock)
                                 foreach my $ckey (sort keys %clackscache) {
                                     $clients{$cid}->{outbuffer} .= "KEYSYNC " . $clackscachetime{$ckey} . " U $ckey=" . $clackscache{$ckey} . "\r\n";
                                 }
@@ -488,10 +498,6 @@ sub run { ## no critic (Subroutines::ProhibitExcessComplexity)
                                     next if(defined($clackscache{$ckey}));
                                     $clients{$cid}->{outbuffer} .= "KEYSYNC " . $clackscachetime{$ckey} . " D $ckey=REMOVED\r\n";
                                 }
-
-                            } else {
-                                print "Interclacks sync lock OFF.\n";
-                                $interclackslock = 0;
                             }
                             $parsedflags{forward_message} = 0; # Don't forward
                             $newflags{return_to_sender} = 0; # Don't return to sender
